package org.wso2.wsas.sample.eventing;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.savan.eventing.client.EventingClient;
import org.apache.savan.eventing.client.EventingClientBean;
import org.apache.savan.eventing.client.SubscriptionStatus;

public class Client {

	final static String SUBSCRIBER_ID = "wso2-svan-subscriber";

	static BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
	static EventingClient eventingClient = null;

	public static void main(String[] args) throws Exception {
		ServiceClient client = null;
		ConfigurationContext confContext = null;
		if (args.length != 2) {
			System.out.println("Usage: Client <event source epr> <event sink epr>");
			return;
		}

		confContext = ConfigurationContextFactory
				.createConfigurationContextFromFileSystem("repository");

		client = new ServiceClient(confContext, null);
		client.engageModule("addressing");
		int option = 0;
		boolean goahead = true;

		while (goahead) {
			displayMenu();
			option = getIntInput();
			switch (option) {
				case 1 :
					startPublishing(client, args[0]);
					break;
				case 2 :
					subscribe(client, args[0], args[1]);
					break;
				case 3 :
					renew();
					break;
				case 4 :
					getStatus();
					break;
				case 5 :
					unsubscribe();
					break;
				case 6 :
					stopPublishing(client, args[0]);
					break;
				case 7 :
					goahead = false;
					break;
				default :
					goahead = false;
			}
		}
	}

	private static void displayMenu() {
		System.out.println("Press 1 to start publishing");
		System.out.println("Press 2 to subscribe");
		System.out.println("Press 3 to renew");
		System.out.println("Press 4 to get status");
		System.out.println("Press 5 to unsubscribe");
		System.out.println("Press 6 to stop publishing");
		System.out.println("Press 7 to Exit");
	}

	private static int getIntInput() throws IOException {
		String option = reader.readLine();
		try {
			return Integer.parseInt(option);
		} catch (NumberFormatException e) {
			// invalid option
			return -1;
		}
	}

	private static void subscribe(ServiceClient client, String publisherEpr, String consumerEpr)
			throws Exception {

		if (eventingClient != null) {
			eventingClient.unsubscribe(SUBSCRIBER_ID);
		}

		Options options = null;
		EventingClientBean bean = null;

		options = new Options();
		options.setTo(new EndpointReference(publisherEpr));
		client.setOptions(options);
		options.setManageSession(true);

		bean = new EventingClientBean();
		bean.setDeliveryEPR(new EndpointReference(consumerEpr));

		eventingClient = new EventingClient(client);
		eventingClient.subscribe(bean, SUBSCRIBER_ID);
	}

	private static void renew() throws Exception {

		if (eventingClient == null) {
			System.out.println("You need to subscribe first");
			return;
		}

		eventingClient.renewSubscription(new Date(110, 12, 12), SUBSCRIBER_ID);
	}

	private static void getStatus() throws Exception {

		SubscriptionStatus status = null;

		if (eventingClient == null) {
			System.out.println("You need to subscribe first");
			return;
		}
		status = eventingClient.getSubscriptionStatus(SUBSCRIBER_ID);
		System.out.println(status.getExpirationValue());
	}

	private static void unsubscribe() throws Exception {
		if (eventingClient != null) {
			eventingClient.unsubscribe(SUBSCRIBER_ID);
		}
	}

	private static void startPublishing(ServiceClient client, String epr) {
		Options options = null;

		options = new Options();
		options.setAction("urn:publish");
		options.setTo(new EndpointReference(epr));
		client.setOptions(options);
		options.setManageSession(true);

		try {
			client.fireAndForget(getPayload("publish"));
		} catch (AxisFault e) {
			e.printStackTrace();
		}
	}

	private static void stopPublishing(ServiceClient client, String epr) {
		Options options = null;

		options = new Options();
		options.setAction("urn:stopPublishing");
		options.setTo(new EndpointReference(epr));
		client.setOptions(options);
		options.setManageSession(true);

		try {
			client.fireAndForget(getPayload("stopPublishing"));
		} catch (AxisFault e) {
			e.printStackTrace();
		}
	}

	private static OMElement getPayload(String operation) {
		OMFactory factory = null;
		OMNamespace ns = null;
		OMElement elem = null;
		factory = OMAbstractFactory.getOMFactory();
		ns = factory.createOMNamespace("http://eventing.sample.wsas.wso2.org", "ns1");
		elem = factory.createOMElement(operation, ns);
		return elem;
	}

}